/*
 * Copyright (C) 2006-2009 Andre Noll <maan@systemlinux.org>
 *
 * Licensed under the GPL v2. For licencing details see COPYING.
 */
#include <stdbool.h>

/** \file net.h exported symbols from net.c */

/**
 * the buffer size of the sun_path component of struct sockaddr_un
 *
 * While glibc doesn't define \p UNIX_PATH_MAX, it
 * documents it has being limited to 108 bytes.
 */
#ifndef UNIX_PATH_MAX
#define UNIX_PATH_MAX 108
#endif

/** \cond Userland defines for Linux DCCP support. */
#ifndef IPPROTO_DCCP
#define IPPROTO_DCCP	33	/**< IANA assigned value */
#define SOCK_DCCP	6	/**< Linux socket type	 */
#define SOL_DCCP        269	/**< Linux socket level	 */
#endif
/** \endcond */


/**
 * Functions to parse and validate (parts of) URLs.
 */
extern char *parse_cidr(const char *cidr,
			char *addr, ssize_t addrlen, int32_t *netmask);
extern char *parse_url(const char *url,
		       char *host, ssize_t hostlen, int32_t *port);
/**
 * Ensure that string conforms to the IPv4 address format.
 *
 * \param address The address string to check.
 *
 * \return 1 if \a address conforms to the IPv4 address format, else 0.
 */
_static_inline_ bool is_valid_ipv4_address(const char *address)
{
	struct in_addr test_it;

	return inet_pton(AF_INET, address, &test_it) != 0;
}

/**
 * Ensure that string conforms to IPv6 address format.
 *
 * \param address The address string to check.
 *
 * \return 1 if string has a valid IPv6 address syntax, 0 if not.
 * \sa RFC 4291
 */
_static_inline_ bool is_valid_ipv6_address(const char *address)
{
	struct in6_addr test_it;

	return inet_pton(AF_INET6, address, &test_it) != 0;
}

/**
 * Generic socket creation (passive and active sockets).
 */
extern int makesock(unsigned l3type, unsigned l4type, int passive,
	            const char *host, unsigned short port_number);
extern struct in_addr extract_v4_addr(const struct sockaddr_storage *ss);

/**
 * Functions to support listening sockets.
 */
/** How many pending connections queue of a listening server will hold. */
#define BACKLOG	10
extern int para_listen(unsigned l3type, unsigned l4type, unsigned short port);

/** Pretty-printing of IPv4/6 socket addresses */
extern char *local_name(int sockfd);
extern char *remote_name(int sockfd);

/** used to crypt the communication between para_server and para_client */
typedef void crypt_function(unsigned long len,
	const unsigned char *indata, unsigned char *outdata, void *private_data);

int send_buffer(int, const char *);
int send_bin_buffer(int, const char *, size_t);
__printf_2_3 int send_va_buffer(int fd, const char *fmt, ...);
int recv_buffer(int fd, char *buf, size_t size);
int recv_bin_buffer(int fd, char *buf, size_t size);
int para_accept(int, void *addr, socklen_t size);
int create_local_socket(const char *name, struct sockaddr_un *unix_addr,
	mode_t mode);
int create_remote_socket(const char *name);
int recv_cred_buffer(int, char *, size_t);
ssize_t send_cred_buffer(int, char*);
int recv_pattern(int fd, const char *pattern, size_t bufsize);
void enable_crypt(int fd, crypt_function *recv_f, crypt_function *send_f,
	void *private_data);
void disable_crypt(int fd);
